"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _pouchdbUtils = require("pouchdb-utils");
var _pouchdbMerge = require("pouchdb-merge");
var _pouchdbJson = require("pouchdb-json");
var _pouchdbBinaryUtils = require("pouchdb-binary-utils");
var _bulkDocs = _interopRequireDefault(require("./bulkDocs.js"));
var _pouchdbErrors = require("pouchdb-errors");
var _constants = require("./constants.js");
var _utils = require("./utils.js");
var _openDatabase = _interopRequireWildcard(require("./openDatabase.js"));
var _debug = require("./debug.js");
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function (e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { default: e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && {}.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n.default = e, t && t.set(e, n), n; }
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
// these indexes cover the ground for most allDocs queries
const BY_SEQ_STORE_DELETED_INDEX_SQL = "CREATE INDEX IF NOT EXISTS 'by-seq-deleted-idx' ON " + _constants.BY_SEQ_STORE + ' (seq, deleted)';
const BY_SEQ_STORE_DOC_ID_REV_INDEX_SQL = "CREATE UNIQUE INDEX IF NOT EXISTS 'by-seq-doc-id-rev' ON " + _constants.BY_SEQ_STORE + ' (doc_id, rev)';
const DOC_STORE_WINNINGSEQ_INDEX_SQL = "CREATE INDEX IF NOT EXISTS 'doc-winningseq-idx' ON " + _constants.DOC_STORE + ' (winningseq)';
const ATTACH_AND_SEQ_STORE_SEQ_INDEX_SQL = "CREATE INDEX IF NOT EXISTS 'attach-seq-seq-idx' ON " + _constants.ATTACH_AND_SEQ_STORE + ' (seq)';
const ATTACH_AND_SEQ_STORE_ATTACH_INDEX_SQL = "CREATE UNIQUE INDEX IF NOT EXISTS 'attach-seq-digest-idx' ON " + _constants.ATTACH_AND_SEQ_STORE + ' (digest, seq)';
const DOC_STORE_AND_BY_SEQ_JOINER = _constants.BY_SEQ_STORE + '.seq = ' + _constants.DOC_STORE + '.winningseq';
const SELECT_DOCS = _constants.BY_SEQ_STORE + '.seq AS seq, ' + _constants.BY_SEQ_STORE + '.deleted AS deleted, ' + _constants.BY_SEQ_STORE + '.json AS data, ' + _constants.BY_SEQ_STORE + '.rev AS rev, ' + _constants.DOC_STORE + '.json AS metadata';
const sqliteChanges = new _pouchdbUtils.changesHandler();
function SqlPouch(opts, cb) {
  // @ts-ignore
  let api = this;
  let db;
  // @ts-ignore
  let txnQueue;
  let instanceId;
  let encoding = 'UTF-8';
  api.auto_compaction = false;
  api._name = opts.name;
  _debug.logger.debug('Creating SqlPouch instance: %s', api._name, opts);
  const sqlOpts = Object.assign({}, opts, {
    name: opts.name + '.sqlite'
  });
  const openDBResult = (0, _openDatabase.default)(sqlOpts);
  if ('db' in openDBResult) {
    db = openDBResult.db;
    txnQueue = openDBResult.transactionQueue;
    setup(cb);
    _debug.logger.debug('Database was opened successfully.', db.getDbPath());
  } else {
    (0, _utils.handleSQLiteError)(openDBResult.error, cb);
  }
  async function transaction(fn) {
    return txnQueue.push(fn);
  }
  async function readTransaction(fn) {
    return txnQueue.pushReadOnly(fn);
  }
  async function setup(callback) {
    await db.transaction(async tx => {
      await Promise.all([checkEncoding(tx), fetchVersion(tx)]);
    });
    callback(null);
  }
  async function checkEncoding(tx) {
    const res = await tx.execute("SELECT HEX('a') AS hex");
    const hex = res.rows[0].hex;
    encoding = hex.length === 2 ? 'UTF-8' : 'UTF-16';
  }
  async function fetchVersion(tx) {
    const sql = 'SELECT sql FROM sqlite_master WHERE tbl_name = ' + _constants.META_STORE;
    const result = await tx.execute(sql, []);
    if (!result.rows?.length) {
      await onGetVersion(tx, 0);
    } else if (!/db_version/.test(result.rows[0].sql)) {
      await tx.execute('ALTER TABLE ' + _constants.META_STORE + ' ADD COLUMN db_version INTEGER');
      await onGetVersion(tx, 1);
    } else {
      const resDBVer = await tx.execute('SELECT db_version FROM ' + _constants.META_STORE);
      const dbVersion = resDBVer.rows[0].db_version;
      await onGetVersion(tx, dbVersion);
    }
  }
  async function onGetVersion(tx, dbVersion) {
    if (dbVersion === 0) {
      await createInitialSchema(tx);
    } else {
      await runMigrations(tx, dbVersion);
    }
  }
  async function createInitialSchema(tx) {
    const meta = 'CREATE TABLE IF NOT EXISTS ' + _constants.META_STORE + ' (dbid, db_version INTEGER)';
    const attach = 'CREATE TABLE IF NOT EXISTS ' + _constants.ATTACH_STORE + ' (digest UNIQUE, escaped TINYINT(1), body BLOB)';
    const attachAndRev = 'CREATE TABLE IF NOT EXISTS ' + _constants.ATTACH_AND_SEQ_STORE + ' (digest, seq INTEGER)';
    const doc = 'CREATE TABLE IF NOT EXISTS ' + _constants.DOC_STORE + ' (id unique, json, winningseq, max_seq INTEGER UNIQUE)';
    const seq = 'CREATE TABLE IF NOT EXISTS ' + _constants.BY_SEQ_STORE + ' (seq INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT, json, deleted TINYINT(1), doc_id, rev)';
    const local = 'CREATE TABLE IF NOT EXISTS ' + _constants.LOCAL_STORE + ' (id UNIQUE, rev, json)';
    await tx.execute(attach);
    await tx.execute(local);
    await tx.execute(attachAndRev);
    await tx.execute(ATTACH_AND_SEQ_STORE_SEQ_INDEX_SQL);
    await tx.execute(ATTACH_AND_SEQ_STORE_ATTACH_INDEX_SQL);
    await tx.execute(doc);
    await tx.execute(DOC_STORE_WINNINGSEQ_INDEX_SQL);
    await tx.execute(seq);
    await tx.execute(BY_SEQ_STORE_DELETED_INDEX_SQL);
    await tx.execute(BY_SEQ_STORE_DOC_ID_REV_INDEX_SQL);
    await tx.execute(meta);
    const initSeq = 'INSERT INTO ' + _constants.META_STORE + ' (db_version, dbid) VALUES (?,?)';
    instanceId = (0, _pouchdbUtils.uuid)();
    const initSeqArgs = [_constants.ADAPTER_VERSION, instanceId];
    await tx.execute(initSeq, initSeqArgs);
    onGetInstanceId();
  }
  async function runMigrations(_tx, dbVersion) {
    // const tasks = [setupDone]
    //
    // let i = dbVersion
    // const nextMigration = (tx: Transaction) => {
    //   tasks[i - 1](tx, nextMigration)
    //   i++
    // }
    // nextMigration(tx)

    const migrated = dbVersion < _constants.ADAPTER_VERSION;
    if (migrated) {
      await db.execute('UPDATE ' + _constants.META_STORE + ' SET db_version = ' + _constants.ADAPTER_VERSION);
    }
    const result = await db.execute('SELECT dbid FROM ' + _constants.META_STORE);
    instanceId = result.rows[0].dbid;
    onGetInstanceId();
  }
  function onGetInstanceId() {
    // Do nothing
  }
  api._remote = false;
  api._id = callback => {
    callback(null, instanceId);
  };
  api._info = callback => {
    readTransaction(async tx => {
      try {
        const seq = await getMaxSeq(tx);
        const docCount = await countDocs(tx);
        callback(null, {
          doc_count: docCount,
          update_seq: seq,
          sqlite_encoding: encoding
        });
      } catch (e) {
        (0, _utils.handleSQLiteError)(e, callback);
      }
    });
  };
  api._bulkDocs = async (req, reqOpts, callback) => {
    _debug.logger.debug('**********bulkDocs!!!!!!!!!!!!!!!!!!!');
    try {
      const response = await (0, _bulkDocs.default)({
        revs_limit: opts.revs_limit
      }, req, reqOpts, api, transaction, sqliteChanges);
      callback(null, response);
    } catch (err) {
      (0, _utils.handleSQLiteError)(err, callback);
    }
  };
  api._get = (id, opts, callback) => {
    _debug.logger.debug('get:', id);
    let doc;
    let metadata;
    const tx = opts.ctx;
    if (!tx) {
      readTransaction(async txn => {
        return new Promise(resolve => {
          api._get(id, Object.assign({
            ctx: txn
          }, opts), (err, response) => {
            callback(err, response);
            resolve();
          });
        });
      });
      return;
    }
    const finish = err => {
      callback(err, {
        doc,
        metadata,
        ctx: tx
      });
    };
    let sql;
    let sqlArgs;
    if (!opts.rev) {
      sql = (0, _utils.select)(SELECT_DOCS, [_constants.DOC_STORE, _constants.BY_SEQ_STORE], DOC_STORE_AND_BY_SEQ_JOINER, _constants.DOC_STORE + '.id=?');
      sqlArgs = [id];
    } else if (opts.latest) {
      latest(tx, id, opts.rev, latestRev => {
        opts.latest = false;
        opts.rev = latestRev;
        api._get(id, opts, callback);
      }, finish);
      return;
    } else {
      sql = (0, _utils.select)(SELECT_DOCS, [_constants.DOC_STORE, _constants.BY_SEQ_STORE], _constants.DOC_STORE + '.id=' + _constants.BY_SEQ_STORE + '.doc_id', [_constants.BY_SEQ_STORE + '.doc_id=?', _constants.BY_SEQ_STORE + '.rev=?']);
      sqlArgs = [id, opts.rev];
    }
    tx.execute(sql, sqlArgs).then(results => {
      if (!results.rows?.length) {
        const missingErr = (0, _pouchdbErrors.createError)(_pouchdbErrors.MISSING_DOC, 'missing');
        return finish(missingErr);
      }
      const item = results.rows[0];
      metadata = (0, _pouchdbJson.safeJsonParse)(item.metadata);
      if (item.deleted && !opts.rev) {
        const deletedErr = (0, _pouchdbErrors.createError)(_pouchdbErrors.MISSING_DOC, 'deleted');
        return finish(deletedErr);
      }
      doc = (0, _utils.unstringifyDoc)(item.data, metadata.id, item.rev);
      finish(null);
    }).catch(e => {
      // createError will throw in RN 0.76.3
      // https://github.com/facebook/hermes/issues/1496
      return finish(e);
    });
  };
  api._allDocs = (opts, callback) => {
    const results = [];
    const start = 'startkey' in opts ? opts.startkey : false;
    const end = 'endkey' in opts ? opts.endkey : false;
    const key = 'key' in opts ? opts.key : false;
    const keys = 'keys' in opts ? opts.keys : false;
    const descending = 'descending' in opts ? opts.descending : false;
    let limit = 'limit' in opts ? opts.limit : -1;
    const offset = 'skip' in opts ? opts.skip : 0;
    const inclusiveEnd = opts.inclusive_end !== false;
    let sqlArgs = [];
    const criteria = [];
    const keyChunks = [];
    if (keys) {
      const destinctKeys = [];
      keys.forEach(key => {
        if (destinctKeys.indexOf(key) === -1) {
          destinctKeys.push(key);
        }
      });
      for (let index = 0; index < destinctKeys.length; index += 999) {
        const chunk = destinctKeys.slice(index, index + 999);
        if (chunk.length > 0) {
          keyChunks.push(chunk);
        }
      }
    } else if (key !== false) {
      criteria.push(_constants.DOC_STORE + '.id = ?');
      sqlArgs.push(key);
    } else if (start !== false || end !== false) {
      if (start !== false) {
        criteria.push(_constants.DOC_STORE + '.id ' + (descending ? '<=' : '>=') + ' ?');
        sqlArgs.push(start);
      }
      if (end !== false) {
        let comparator = descending ? '>' : '<';
        if (inclusiveEnd) {
          comparator += '=';
        }
        criteria.push(_constants.DOC_STORE + '.id ' + comparator + ' ?');
        sqlArgs.push(end);
      }
      if (key !== false) {
        criteria.push(_constants.DOC_STORE + '.id = ?');
        sqlArgs.push(key);
      }
    }
    if (!keys) {
      criteria.push(_constants.BY_SEQ_STORE + '.deleted = 0');
    }
    readTransaction(async tx => {
      const processResult = (rows, results, keys) => {
        for (let i = 0, l = rows.length; i < l; i++) {
          const item = rows[i];
          const metadata = (0, _pouchdbJson.safeJsonParse)(item.metadata);
          const id = metadata.id;
          const data = (0, _utils.unstringifyDoc)(item.data, id, item.rev);
          const winningRev = data._rev;
          const doc = {
            id: id,
            key: id,
            value: {
              rev: winningRev
            }
          };
          if (opts.include_docs) {
            doc.doc = data;
            doc.doc._rev = winningRev;
            if (opts.conflicts) {
              const conflicts = (0, _pouchdbMerge.collectConflicts)(metadata);
              if (conflicts.length) {
                doc.doc._conflicts = conflicts;
              }
            }
            fetchAttachmentsIfNecessary(doc.doc, opts, api, tx);
          }
          if (item.deleted) {
            if (keys) {
              doc.value.deleted = true;
              doc.doc = null;
            } else {
              continue;
            }
          }
          if (!keys) {
            results.push(doc);
          } else {
            let index = keys.indexOf(id);
            do {
              results[index] = doc;
              index = keys.indexOf(id, index + 1);
            } while (index > -1 && index < keys.length);
          }
        }
        if (keys) {
          keys.forEach((key, index) => {
            if (!results[index]) {
              results[index] = {
                key: key,
                error: 'not_found'
              };
            }
          });
        }
      };
      try {
        const totalRows = await countDocs(tx);
        const updateSeq = opts.update_seq ? await getMaxSeq(tx) : undefined;
        if (limit === 0) {
          limit = 1;
        }
        if (keys) {
          let finishedCount = 0;
          const allRows = [];
          for (const keyChunk of keyChunks) {
            sqlArgs = [];
            criteria.length = 0;
            let bindingStr = '';
            keyChunk.forEach(() => {
              bindingStr += '?,';
            });
            bindingStr = bindingStr.substring(0, bindingStr.length - 1);
            criteria.push(_constants.DOC_STORE + '.id IN (' + bindingStr + ')');
            sqlArgs = sqlArgs.concat(keyChunk);
            const sql = (0, _utils.select)(SELECT_DOCS, [_constants.DOC_STORE, _constants.BY_SEQ_STORE], DOC_STORE_AND_BY_SEQ_JOINER, criteria, _constants.DOC_STORE + '.id ' + (descending ? 'DESC' : 'ASC')) + ' LIMIT ' + limit + ' OFFSET ' + offset;
            const result = await tx.execute(sql, sqlArgs);
            finishedCount++;
            if (result.rows) {
              for (let index = 0; index < result.rows.length; index++) {
                allRows.push(result.rows[index]);
              }
            }
            if (finishedCount === keyChunks.length) {
              processResult(allRows, results, keys);
            }
          }
        } else {
          const sql = (0, _utils.select)(SELECT_DOCS, [_constants.DOC_STORE, _constants.BY_SEQ_STORE], DOC_STORE_AND_BY_SEQ_JOINER, criteria, _constants.DOC_STORE + '.id ' + (descending ? 'DESC' : 'ASC')) + ' LIMIT ' + limit + ' OFFSET ' + offset;
          const result = await tx.execute(sql, sqlArgs);
          const rows = [];
          if (result.rows) {
            for (let index = 0; index < result.rows.length; index++) {
              rows.push(result.rows[index]);
            }
          }
          processResult(rows, results, keys);
        }
        const returnVal = {
          total_rows: totalRows,
          offset: opts.skip,
          rows: results
        };
        if (opts.update_seq) {
          returnVal.update_seq = updateSeq;
        }
        callback(null, returnVal);
      } catch (e) {
        (0, _utils.handleSQLiteError)(e, callback);
      }
    });
  };
  api._changes = opts => {
    opts = (0, _pouchdbUtils.clone)(opts);
    if (opts.continuous) {
      const id = api._name + ':' + (0, _pouchdbUtils.uuid)();
      sqliteChanges.addListener(api._name, id, api, opts);
      sqliteChanges.notify(api._name);
      return {
        cancel: () => {
          sqliteChanges.removeListener(api._name, id);
        }
      };
    }
    const descending = opts.descending;
    opts.since = opts.since && !descending ? opts.since : 0;
    let limit = 'limit' in opts ? opts.limit : -1;
    if (limit === 0) {
      limit = 1;
    }
    const results = [];
    let numResults = 0;
    const fetchChanges = () => {
      const selectStmt = _constants.DOC_STORE + '.json AS metadata, ' + _constants.DOC_STORE + '.max_seq AS maxSeq, ' + _constants.BY_SEQ_STORE + '.json AS winningDoc, ' + _constants.BY_SEQ_STORE + '.rev AS winningRev ';
      const from = _constants.DOC_STORE + ' JOIN ' + _constants.BY_SEQ_STORE;
      const joiner = _constants.DOC_STORE + '.id=' + _constants.BY_SEQ_STORE + '.doc_id' + ' AND ' + _constants.DOC_STORE + '.winningseq=' + _constants.BY_SEQ_STORE + '.seq';
      const criteria = ['maxSeq > ?'];
      const sqlArgs = [opts.since];
      if (opts.doc_ids) {
        criteria.push(_constants.DOC_STORE + '.id IN ' + (0, _utils.qMarks)(opts.doc_ids.length));
        sqlArgs.push(...opts.doc_ids);
      }
      const orderBy = 'maxSeq ' + (descending ? 'DESC' : 'ASC');
      let sql = (0, _utils.select)(selectStmt, from, joiner, criteria, orderBy);
      const filter = (0, _pouchdbUtils.filterChange)(opts);
      if (!opts.view && !opts.filter) {
        sql += ' LIMIT ' + limit;
      }
      let lastSeq = opts.since || 0;
      readTransaction(async tx => {
        try {
          const result = await tx.execute(sql, sqlArgs);
          if (result.rows) {
            for (let i = 0, l = result.rows.length; i < l; i++) {
              const item = result.rows[i];
              const metadata = (0, _pouchdbJson.safeJsonParse)(item.metadata);
              lastSeq = item.maxSeq;
              const doc = (0, _utils.unstringifyDoc)(item.winningDoc, metadata.id, item.winningRev);
              const change = opts.processChange(doc, metadata, opts);
              change.seq = item.maxSeq;
              const filtered = filter(change);
              if (typeof filtered === 'object') {
                return opts.complete(filtered);
              }
              if (filtered) {
                numResults++;
                if (opts.return_docs) {
                  results.push(change);
                }
                if (opts.attachments && opts.include_docs) {
                  fetchAttachmentsIfNecessary(doc, opts, api, tx, () => opts.onChange(change));
                } else {
                  opts.onChange(change);
                }
              }
              if (numResults === limit) {
                break;
              }
            }
          }
          if (!opts.continuous) {
            opts.complete(null, {
              results,
              last_seq: lastSeq
            });
          }
        } catch (e) {
          (0, _utils.handleSQLiteError)(e, opts.complete);
        }
      });
    };
    fetchChanges();
  };
  api._close = callback => {
    (0, _openDatabase.closeDB)(api._name);
    callback();
  };
  api._getAttachment = (_docId, _attachId, attachment, opts, callback) => {
    let res;
    const tx = opts.ctx;
    const digest = attachment.digest;
    const type = attachment.content_type;
    const sql = 'SELECT escaped, body AS body FROM ' + _constants.ATTACH_STORE + ' WHERE digest=?';
    tx.execute(sql, [digest]).then(result => {
      const item = result.rows[0];
      const data = item.body;
      if (opts.binary) {
        res = (0, _pouchdbBinaryUtils.binaryStringToBlobOrBuffer)(data, type);
      } else {
        res = (0, _pouchdbBinaryUtils.btoa)(data);
      }
      callback(null, res);
    });
  };
  api._getRevisionTree = (docId, callback) => {
    readTransaction(async tx => {
      const sql = 'SELECT json AS metadata FROM ' + _constants.DOC_STORE + ' WHERE id = ?';
      const result = await tx.execute(sql, [docId]);
      if (!result.rows?.length) {
        callback((0, _pouchdbErrors.createError)(_pouchdbErrors.MISSING_DOC));
      } else {
        const data = (0, _pouchdbJson.safeJsonParse)(result.rows[0].metadata);
        callback(null, data.rev_tree);
      }
    });
  };
  api._doCompaction = (docId, revs, callback) => {
    if (!revs.length) {
      return callback();
    }
    transaction(async tx => {
      try {
        let sql = 'SELECT json AS metadata FROM ' + _constants.DOC_STORE + ' WHERE id = ?';
        const result = await tx.execute(sql, [docId]);
        const metadata = (0, _pouchdbJson.safeJsonParse)(result.rows[0].metadata);
        (0, _pouchdbMerge.traverseRevTree)(metadata.rev_tree, (_isLeaf, pos, revHash, _ctx, opts) => {
          const rev = pos + '-' + revHash;
          if (revs.indexOf(rev) !== -1) {
            opts.status = 'missing';
          }
        });
        sql = 'UPDATE ' + _constants.DOC_STORE + ' SET json = ? WHERE id = ?';
        await tx.execute(sql, [(0, _pouchdbJson.safeJsonStringify)(metadata), docId]);
        (0, _utils.compactRevs)(revs, docId, tx);
      } catch (e) {
        (0, _utils.handleSQLiteError)(e, callback);
      }
      callback();
    });
  };
  api._getLocal = (id, callback) => {
    readTransaction(async tx => {
      try {
        const sql = 'SELECT json, rev FROM ' + _constants.LOCAL_STORE + ' WHERE id=?';
        const res = await tx.execute(sql, [id]);
        if (res.rows?.length) {
          const item = res.rows[0];
          const doc = (0, _utils.unstringifyDoc)(item.json, id, item.rev);
          callback(null, doc);
        } else {
          callback((0, _pouchdbErrors.createError)(_pouchdbErrors.MISSING_DOC));
        }
      } catch (e) {
        (0, _utils.handleSQLiteError)(e, callback);
      }
    });
  };
  api._putLocal = (doc, opts, callback) => {
    if (typeof opts === 'function') {
      callback = opts;
      opts = {};
    }
    delete doc._revisions;
    const oldRev = doc._rev;
    const id = doc._id;
    let newRev;
    if (!oldRev) {
      newRev = doc._rev = '0-1';
    } else {
      newRev = doc._rev = '0-' + (parseInt(oldRev.split('-')[1], 10) + 1);
    }
    const json = (0, _utils.stringifyDoc)(doc);
    let ret;
    const putLocal = async tx => {
      try {
        let sql;
        let values;
        if (oldRev) {
          sql = 'UPDATE ' + _constants.LOCAL_STORE + ' SET rev=?, json=? WHERE id=? AND rev=?';
          values = [newRev, json, id, oldRev];
        } else {
          sql = 'INSERT INTO ' + _constants.LOCAL_STORE + ' (id, rev, json) VALUES (?,?,?)';
          values = [id, newRev, json];
        }
        const res = await tx.execute(sql, values);
        if (res.rowsAffected) {
          ret = {
            ok: true,
            id: id,
            rev: newRev
          };
          callback(null, ret);
        } else {
          callback((0, _pouchdbErrors.createError)(_pouchdbErrors.REV_CONFLICT));
        }
      } catch (e) {
        (0, _utils.handleSQLiteError)(e, callback);
      }
    };
    if (opts.ctx) {
      putLocal(opts.ctx);
    } else {
      transaction(putLocal);
    }
  };
  api._removeLocal = (doc, opts, callback) => {
    if (typeof opts === 'function') {
      callback = opts;
      opts = {};
    }
    let ret;
    const removeLocal = async tx => {
      try {
        const sql = 'DELETE FROM ' + _constants.LOCAL_STORE + ' WHERE id=? AND rev=?';
        const params = [doc._id, doc._rev];
        const res = await tx.execute(sql, params);
        if (!res.rowsAffected) {
          return callback((0, _pouchdbErrors.createError)(_pouchdbErrors.MISSING_DOC));
        }
        ret = {
          ok: true,
          id: doc._id,
          rev: '0-0'
        };
        callback(null, ret);
      } catch (e) {
        (0, _utils.handleSQLiteError)(e, callback);
      }
    };
    if (opts.ctx) {
      removeLocal(opts.ctx);
    } else {
      transaction(removeLocal);
    }
  };
  api._destroy = (_opts, callback) => {
    sqliteChanges.removeAllListeners(api._name);
    transaction(async tx => {
      try {
        const stores = [_constants.DOC_STORE, _constants.BY_SEQ_STORE, _constants.ATTACH_STORE, _constants.META_STORE, _constants.LOCAL_STORE, _constants.ATTACH_AND_SEQ_STORE];
        stores.forEach(store => {
          tx.execute('DROP TABLE IF EXISTS ' + store, []);
        });
        callback(null, {
          ok: true
        });
      } catch (e) {
        (0, _utils.handleSQLiteError)(e, callback);
      }
    });
  };
  function fetchAttachmentsIfNecessary(doc, opts, api, txn, cb) {
    const attachments = Object.keys(doc._attachments || {});
    if (!attachments.length) {
      return cb && cb();
    }
    let numDone = 0;
    const checkDone = () => {
      if (++numDone === attachments.length && cb) {
        cb();
      }
    };
    const fetchAttachment = (doc, att) => {
      const attObj = doc._attachments[att];
      const attOpts = {
        binary: opts.binary,
        ctx: txn
      };
      api._getAttachment(doc._id, att, attObj, attOpts, (_, data) => {
        doc._attachments[att] = Object.assign((0, _pouchdbUtils.pick)(attObj, ['digest', 'content_type']), {
          data
        });
        checkDone();
      });
    };
    attachments.forEach(att => {
      if (opts.attachments && opts.include_docs) {
        fetchAttachment(doc, att);
      } else {
        doc._attachments[att].stub = true;
        checkDone();
      }
    });
  }
  async function getMaxSeq(tx) {
    const sql = 'SELECT MAX(seq) AS seq FROM ' + _constants.BY_SEQ_STORE;
    const res = await tx.execute(sql, []);
    const updateSeq = res.rows[0].seq || 0;
    return updateSeq;
  }
  async function countDocs(tx) {
    const sql = (0, _utils.select)('COUNT(' + _constants.DOC_STORE + ".id) AS 'num'", [_constants.DOC_STORE, _constants.BY_SEQ_STORE], DOC_STORE_AND_BY_SEQ_JOINER, _constants.BY_SEQ_STORE + '.deleted=0');
    const result = await tx.execute(sql, []);
    return result.rows[0].num || 0;
  }
  async function latest(tx, id, rev, callback, finish) {
    const sql = (0, _utils.select)(SELECT_DOCS, [_constants.DOC_STORE, _constants.BY_SEQ_STORE], DOC_STORE_AND_BY_SEQ_JOINER, _constants.DOC_STORE + '.id=?');
    const sqlArgs = [id];
    const results = await tx.execute(sql, sqlArgs);
    if (!results.rows?.length) {
      const err = (0, _pouchdbErrors.createError)(_pouchdbErrors.MISSING_DOC, 'missing');
      return finish(err);
    }
    const item = results.rows[0];
    const metadata = (0, _pouchdbJson.safeJsonParse)(item.metadata);
    callback((0, _pouchdbMerge.latest)(rev, metadata));
  }
}
var _default = exports.default = SqlPouch;
//# sourceMappingURL=core.js.map