"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.closeDB = closeDB;
exports.default = void 0;
var _opSqlite = require("@op-engineering/op-sqlite");
var _transactionQueue = require("./transactionQueue.js");
const cachedDatabases = new Map();
function openDBSafely(opts) {
  try {
    const db = (0, _opSqlite.open)(opts);
    const transactionQueue = new _transactionQueue.TransactionQueue(db);
    return {
      db,
      transactionQueue
    };
  } catch (err) {
    return {
      error: err
    };
  }
}
function openDB(opts) {
  let cachedResult = cachedDatabases.get(opts.name);
  if (!cachedResult) {
    cachedResult = openDBSafely(opts);
    cachedDatabases.set(opts.name, cachedResult);
  }
  return cachedResult;
}
function closeDB(name) {
  const cachedResult = cachedDatabases.get(name);
  if (cachedResult) {
    if ('db' in cachedResult) {
      cachedResult.db.close();
    }
    cachedDatabases.delete(name);
  }
}
var _default = exports.default = openDB;
//# sourceMappingURL=openDatabase.js.map