"use strict";

import { open } from '@op-engineering/op-sqlite';
import { TransactionQueue } from "./transactionQueue.js";
const cachedDatabases = new Map();
function openDBSafely(opts) {
  try {
    const db = open(opts);
    const transactionQueue = new TransactionQueue(db);
    return {
      db,
      transactionQueue
    };
  } catch (err) {
    return {
      error: err
    };
  }
}
function openDB(opts) {
  let cachedResult = cachedDatabases.get(opts.name);
  if (!cachedResult) {
    cachedResult = openDBSafely(opts);
    cachedDatabases.set(opts.name, cachedResult);
  }
  return cachedResult;
}
export function closeDB(name) {
  const cachedResult = cachedDatabases.get(name);
  if (cachedResult) {
    if ('db' in cachedResult) {
      cachedResult.db.close();
    }
    cachedDatabases.delete(name);
  }
}
export default openDB;
//# sourceMappingURL=openDatabase.js.map